@@ -14,6 +14,7 @@
 #    under the License.
 from cursive import exception as cursive_exception
 import glance_store
+from glance_store import backend
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
@@ -67,6 +68,23 @@ class ImageDataController(object):
                     'e': encodeutils.exception_to_unicode(e)})
             LOG.exception(msg)
 
+    def _unstage(self, image_repo, image, staging_store):
+        """
+        Restore the image to queued status and remove data from staging.
+
+        :param image_repo: The instance of ImageRepo
+        :param image: The image will be restored
+        :param staging_store: The store used for staging
+        """
+        loc = glance_store.location.get_location_from_uri(str(
+            CONF.node_staging_uri + '/' + image.image_id))
+        try:
+            staging_store.delete(loc)
+        except glance_store.exceptions.NotFound:
+            pass
+        finally:
+            self._restore(image_repo, image)
+
     def _delete(self, image_repo, image):
         """Delete the image.
 
@@ -246,7 +264,82 @@ class ImageDataController(object):
 
         except Exception as e:
             with excutils.save_and_reraise_exception():
-                LOG.exception(_LE("Failed to upload image data due to "
+                LOG.error(_LE("Failed to upload image data due to "
+                              "internal error"))
+                self._restore(image_repo, image)
+
+    @utils.mutating
+    def stage(self, req, image_id, data, size):
+        image_repo = self.gateway.get_repo(req.context)
+        image = None
+
+        # NOTE(jokke): this is horrible way to do it but as long as
+        # glance_store is in a shape it is, the only way. Don't hold me
+        # accountable for it.
+        def _build_staging_store():
+            conf = cfg.ConfigOpts()
+            backend.register_opts(conf)
+            conf.set_override('filesystem_store_datadir',
+                              CONF.node_staging_uri[7:],
+                              group='glance_store')
+            staging_store = backend._load_store(conf, 'file')
+
+            try:
+                staging_store.configure()
+            except AttributeError:
+                msg = _("'node_staging_uri' is not set correctly. Could not "
+                        "load staging store.")
+                raise exception.BadStoreUri(message=msg)
+            return staging_store
+
+        staging_store = _build_staging_store()
+
+        try:
+            image = image_repo.get(image_id)
+            image.status = 'uploading'
+            image_repo.save(image, from_state='queued')
+            try:
+                staging_store.add(image_id, data, 0)
+            except glance_store.Duplicate as e:
+                msg = _("The image %s has data on staging") % image_id
+                raise webob.exc.HTTPConflict(explanation=msg)
+                self._restore(image_repo, image)
+
+        except glance_store.StorageFull as e:
+            msg = _("Image storage media "
+                    "is full: %s") % encodeutils.exception_to_unicode(e)
+            LOG.error(msg)
+            self._unstage(image_repo, image, staging_store)
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
+                                                      request=req)
+
+        except exception.StorageQuotaFull as e:
+            msg = _("Image exceeds the storage "
+                    "quota: %s") % encodeutils.exception_to_unicode(e)
+            LOG.debug(msg)
+            self._unstage(image_repo, image, staging_store)
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
+                                                      request=req)
+
+        except exception.ImageSizeLimitExceeded as e:
+            msg = _("The incoming image is "
+                    "too large: %s") % encodeutils.exception_to_unicode(e)
+            LOG.debug(msg)
+            self._unstage(image_repo, image, staging_store)
+            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
+                                                      request=req)
+
+        except glance_store.StorageWriteDenied as e:
+            msg = _("Insufficient permissions on image "
+                    "storage media: %s") % encodeutils.exception_to_unicode(e)
+            LOG.error(msg)
+            self._unstage(image_repo, image)
+            raise webob.exc.HTTPServiceUnavailable(explanation=msg,
+                                                   request=req)
+
+        except Exception as e:
+            with excutils.save_and_reraise_exception():
+                LOG.exception(_LE("Failed to stage image data due to "
                                   "internal error"))
                 self._restore(image_repo, image)
 
@@ -275,6 +368,24 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         except exception.InvalidContentType as e:
             raise webob.exc.HTTPUnsupportedMediaType(explanation=e.msg)
 
+        if self.is_valid_encoding(request) and self.is_valid_method(request):
+            request.is_body_readable = True
+
+        image_size = request.content_length or None
+        return {'size': image_size, 'data': request.body_file}
+
+    def stage(self, request):
+        if not CONF.enable_image_import:
+            msg = _("Image import is not supported at this site.")
+            raise webob.exc.HTTPNotFound(explanation=msg)
+        try:
+            request.get_content_type(('application/octet-stream',))
+        except exception.InvalidContentType as e:
+            raise webob.exc.HTTPUnsupportedMediaType(explanation=e.msg)
+
+        if self.is_valid_encoding(request) and self.is_valid_method(request):
+            request.is_body_readable = True
+
         image_size = request.content_length or None
         return {'size': image_size, 'data': request.body_file}
 
@@ -282,16 +393,43 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 class ResponseSerializer(wsgi.JSONResponseSerializer):
 
     def download(self, response, image):
+
         offset, chunk_size = 0, None
-        # NOTE(dharinic): In case of a malformed content range,
+        # NOTE(dharinic): In case of a malformed range header,
         # glance/common/wsgi.py will raise HTTPRequestRangeNotSatisfiable
         # (setting status_code to 416)
-        range_val = response.request.get_content_range(image.size)
+        range_val = response.request.get_range_from_request(image.size)
 
         if range_val:
-            # NOTE(flaper87): if not present, both, start
-            # and stop, will be None.
-            if range_val.start is not None and range_val.stop is not None:
+            if isinstance(range_val, webob.byterange.Range):
+                response_end = image.size - 1
+                # NOTE(dharinic): webob parsing is zero-indexed.
+                # i.e.,to download first 5 bytes of a 10 byte image,
+                # request should be "bytes=0-4" and the response would be
+                # "bytes 0-4/10".
+                # Range if validated, will never have 'start' object as None.
+                if range_val.start >= 0:
+                    offset = range_val.start
+                else:
+                    # NOTE(dharinic): Negative start values needs to be
+                    # processed to allow suffix-length for Range request
+                    # like "bytes=-2" as per rfc7233.
+                    if abs(range_val.start) < image.size:
+                        offset = image.size + range_val.start
+
+                if range_val.end is not None and range_val.end < image.size:
+                    chunk_size = range_val.end - offset
+                    response_end = range_val.end - 1
+                else:
+                    chunk_size = image.size - offset
+
+            # NOTE(dharinic): For backward compatibility reasons, we maintain
+            # support for 'Content-Range' in requests even though it's not
+            # correct to use it in requests.
+            elif isinstance(range_val, webob.byterange.ContentRange):
+                response_end = range_val.stop - 1
+                # NOTE(flaper87): if not present, both, start
+                # and stop, will be None.
                 offset = range_val.start
                 chunk_size = range_val.stop - offset
 
@@ -308,7 +446,12 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
             # NOTE(dharinic): In case of a full image download, when
             # chunk_size was none, reset it to image.size to set the
             # response header's Content-Length.
-            if not chunk_size:
+            if chunk_size is not None:
+                response.headers['Content-Range'] = 'bytes %s-%s/%s'\
+                                                    % (offset,
+                                                       response_end,
+                                                       image.size)
+            else:
                 chunk_size = image.size
         except glance_store.NotFound as e:
             raise webob.exc.HTTPNoContent(explanation=e.msg)
@@ -332,6 +475,9 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
     def upload(self, response, result):
         response.status_int = 204
 
+    def stage(self, response, result):
+        response.status_int = 204
+
 
 def create_resource():
     """Image data resource factory method"""
