@@ -25,6 +25,7 @@ from six.moves import http_client as http
 import six.moves.urllib.parse as urlparse
 import webob.exc
 
+from glance.api import common
 from glance.api import policy
 from glance.common import exception
 from glance.common import location_strategy
@@ -87,6 +88,28 @@ class ImagesController(object):
 
         return image
 
+    @utils.mutating
+    def import_image(self, req, image_id, body):
+        task_factory = self.gateway.get_task_factory(req.context)
+        executor_factory = self.gateway.get_task_executor_factory(req.context)
+        task_repo = self.gateway.get_task_repo(req.context)
+
+        task_input = {'image_id': image_id}
+
+        try:
+            import_task = task_factory.new_task(task_type='api_image_import',
+                                                owner=req.context.owner,
+                                                task_input=task_input)
+            task_repo.add(import_task)
+            task_executor = executor_factory.new_task_executor(req.context)
+            pool = common.get_thread_pool("tasks_eventlet_pool")
+            pool.spawn_n(import_task.run, task_executor)
+        except exception.Forbidden as e:
+            LOG.debug("User not permitted to create image import task.")
+            raise webob.exc.HTTPForbidden(explanation=e.msg)
+
+        return image_id
+
     def index(self, req, marker=None, limit=None, sort_key=None,
               sort_dir=None, filters=None, member_status='accepted'):
         sort_key = ['created_at'] if not sort_key else sort_key
@@ -98,6 +121,15 @@ class ImagesController(object):
             filters = {}
         filters['deleted'] = False
 
+        protected = filters.get('protected')
+        if protected is not None:
+            if protected not in ['true', 'false']:
+                message = _("Invalid value '%s' for 'protected' filter."
+                            " Valid values are 'true' or 'false'.") % protected
+                raise webob.exc.HTTPBadRequest(explanation=message)
+            # ensure the type of protected is boolean
+            filters['protected'] = protected == 'true'
+
         if limit is None:
             limit = CONF.limit_param_default
         limit = min(CONF.api_limit_max, limit)
@@ -635,7 +667,7 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
             if visibility not in ['community', 'public', 'private', 'shared']:
                 msg = _('Invalid visibility value: %s') % visibility
                 raise webob.exc.HTTPBadRequest(explanation=msg)
-        changes_since = filters.get('changes-since', None)
+        changes_since = filters.get('changes-since')
         if changes_since:
             msg = _('The "changes-since" filter is no longer available on v2.')
             raise webob.exc.HTTPBadRequest(explanation=msg)
@@ -735,6 +767,31 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
         return query_params
 
+    def _validate_import_body(self, body):
+        # TODO(rosmaita): do schema validation of body instead
+        # of this ad-hoc stuff
+        try:
+            method = body['method']
+        except KeyError:
+            msg = _("Import request requires a 'method' field.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        try:
+            method_name = method['name']
+        except KeyError:
+            msg = _("Import request requires a 'name' field.")
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+        if method_name != 'glance-direct':
+            msg = _("Unknown import method name '%s'.") % method_name
+            raise webob.exc.HTTPBadRequest(explanation=msg)
+
+    def import_image(self, request):
+        if not CONF.enable_image_import:
+            msg = _("Image import is not supported at this site.")
+            raise webob.exc.HTTPNotFound(explanation=msg)
+        body = self._get_request_body(request)
+        self._validate_import_body(body)
+        return {'body': body}
+
 
 class ResponseSerializer(wsgi.JSONResponseSerializer):
     def __init__(self, schema=None):
@@ -807,6 +864,12 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
         response.status_int = http.CREATED
         self.show(response, image)
         response.location = self._get_image_href(image)
+        # TODO(jokke): make this configurable when swift-local is implemented
+        # and remove the if statement with the config option.
+        if CONF.enable_image_import:
+            import_methods = ("OpenStack-image-import-methods",
+                              "glance-direct")
+            response.headerlist.append(import_methods)
 
     def show(self, response, image):
         image_view = self._format_image(image)
@@ -842,6 +905,9 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
     def delete(self, response, result):
         response.status_int = http.NO_CONTENT
 
+    def import_image(self, response, result):
+        response.status_int = http.ACCEPTED
+
 
 def get_base_properties():
     return {
