@@ -206,9 +206,11 @@ class NamespaceController(object):
                 namespace_repo.remove(namespace_obj)
                 LOG.debug("Cleaned up namespace %(namespace)s ",
                           {'namespace': namespace.namespace})
-            except exception:
-                msg = (_LE("Failed to delete namespace %(namespace)s ") %
-                       {'namespace': namespace.namespace})
+            except Exception as e:
+                msg = (_LE("Failed to delete namespace %(namespace)s."
+                           "Exception: %(exception)s"),
+                       {'namespace': namespace.namespace,
+                        'exception': encodeutils.exception_to_unicode(e)})
                 LOG.error(msg)
 
     def show(self, req, namespace, filters=None):
@@ -547,6 +549,9 @@ class ResponseSerializer(wsgi.JSONResponseSerializer):
     def delete_properties(self, response, result):
         response.status_int = http.NO_CONTENT
 
+    def delete_tags(self, response, result):
+        response.status_int = http.NO_CONTENT
+
     def __render(self, json_data, response, response_status=None):
         body = jsonutils.dumps(json_data, ensure_ascii=False)
         response.unicode_body = six.text_type(body)
