@@ -292,7 +292,7 @@ def env(*vars, **kwargs):
     returns the default defined in kwargs.
     """
     for v in vars:
-        value = os.environ.get(v, None)
+        value = os.environ.get(v)
         if value:
             return value
     return kwargs.get('default', '')
@@ -516,7 +516,7 @@ Commands:
             print("Completed in %-0.4f sec." % (end_time - start_time))
         sys.exit(result)
     except (RuntimeError, NotImplementedError) as e:
-        print("ERROR: ", e)
+        sys.exit("ERROR: %s" % e)
 
 if __name__ == '__main__':
     main()
