@@ -44,7 +44,8 @@ import six
 from webob import exc
 
 from glance.common import exception
-from glance.i18n import _, _LE
+from glance.common import timeutils
+from glance.i18n import _, _LE, _LW
 
 CONF = cfg.CONF
 
@@ -421,13 +422,14 @@ def validate_key_cert(key_file, cert_file):
         data = encodeutils.to_utf8(data)
         digest = CONF.digest_algorithm
         if digest == 'sha1':
-            LOG.warn('The FIPS (FEDERAL INFORMATION PROCESSING STANDARDS)'
-                     ' state that the SHA-1 is not suitable for'
-                     ' general-purpose digital signature applications (as'
-                     ' specified in FIPS 186-3) that require 112 bits of'
-                     ' security. The default value is sha1 in Kilo for a'
-                     ' smooth upgrade process, and it will be updated'
-                     ' with sha256 in next release(L).')
+            LOG.warn(
+                _LW('The FIPS (FEDERAL INFORMATION PROCESSING STANDARDS)'
+                    ' state that the SHA-1 is not suitable for'
+                    ' general-purpose digital signature applications (as'
+                    ' specified in FIPS 186-3) that require 112 bits of'
+                    ' security. The default value is sha1 in Kilo for a'
+                    ' smooth upgrade process, and it will be updated'
+                    ' with sha256 in next release(L).'))
         out = crypto.sign(key, data, digest)
         crypto.verify(cert, out, data, digest)
     except crypto.Error as ce:
@@ -548,31 +550,6 @@ def no_4byte_params(f):
     return wrapper
 
 
-def validate_mysql_int(*args, **kwargs):
-    """
-    Make sure that all arguments are less than 2 ** 31 - 1.
-
-    This limitation is introduced because mysql stores INT in 4 bytes.
-    If the validation fails for some argument, exception.Invalid is raised with
-    appropriate information.
-    """
-    max_int = (2 ** 31) - 1
-    for param in args:
-        if param > max_int:
-            msg = _("Value %(value)d out of range, "
-                    "must not exceed %(max)d") % {"value": param,
-                                                  "max": max_int}
-            raise exception.Invalid(msg)
-
-    for param_str in kwargs:
-        param = kwargs.get(param_str)
-        if param and param > max_int:
-            msg = _("'%(param)s' value out of range, "
-                    "must not exceed %(max)d") % {"param": param_str,
-                                                  "max": max_int}
-            raise exception.Invalid(msg)
-
-
 def stash_conf_values():
     """
     Make a copy of some of the current global CONF's settings.
@@ -602,8 +579,17 @@ def split_filter_op(expression):
     """
     left, sep, right = expression.partition(':')
     if sep:
-        op = left
-        threshold = right
+        # If the expression is a date of the format ISO 8601 like
+        # CCYY-MM-DDThh:mm:ss+hh:mm and has no operator, it should
+        # not be partitioned, and a default operator of eq should be
+        # assumed.
+        try:
+            timeutils.parse_isotime(expression)
+            op = 'eq'
+            threshold = expression
+        except ValueError:
+            op = left
+            threshold = right
     else:
         op = 'eq'  # default operator
         threshold = left
@@ -666,7 +652,7 @@ def evaluate_filter_op(value, operator, threshold):
     :param operator: any supported filter operation
     :param threshold: to compare value against, as right side of expression
 
-    :raises: InvalidFilterOperatorValue if an unknown operator is provided
+    :raises InvalidFilterOperatorValue: if an unknown operator is provided
 
     :returns: boolean result of applied comparison
 
