@@ -28,7 +28,6 @@ import signal
 import sys
 import time
 
-import eventlet
 from eventlet.green import socket
 from eventlet.green import ssl
 import eventlet.greenio
@@ -41,7 +40,6 @@ from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 from oslo_utils import strutils
 from osprofiler import opts as profiler_opts
-import routes
 import routes.middleware
 import six
 import webob.dec
@@ -56,16 +54,9 @@ from glance.i18n import _, _LE, _LI, _LW
 
 
 bind_opts = [
-    # NOTE(dharinic): Since ``bind_host`` accepts symbolic names
-    # mapped to IP address apart from IP addresses, we use
-    # ``StrOpt`` as against ``IPOpt`` for backward compatibility
-    # reasons. We do not use ``HostnameOpt`` as it would prevent the
-    # opt from accepting IPv6 addresses.
-    # TODO(dharinic): Change the Opt type upon resolution of
-    # bug-1619044
-    cfg.StrOpt('bind_host',
-               default='0.0.0.0',
-               help=_("""
+    cfg.HostAddressOpt('bind_host',
+                       default='0.0.0.0',
+                       help=_("""
 IP address to bind the glance servers to.
 
 Provide an IP address to bind the glance server to. The default
@@ -943,6 +934,26 @@ class Request(webob.Request):
                 environ['wsgi.url_scheme'] = scheme
         super(Request, self).__init__(environ, *args, **kwargs)
 
+    @property
+    def params(self):
+        """Override params property of webob.request.BaseRequest.
+
+        Added an 'encoded_params' attribute in case of PY2 to avoid
+        encoding values in next subsequent calls to the params property.
+        """
+        if six.PY2:
+            encoded_params = getattr(self, 'encoded_params', None)
+            if encoded_params is None:
+                params = super(Request, self).params
+                params_dict = multidict.MultiDict()
+                for key, value in params.items():
+                    params_dict.add(key, encodeutils.safe_encode(value))
+
+                setattr(self, 'encoded_params',
+                        multidict.NestedMultiDict(params_dict))
+            return self.encoded_params
+        return super(Request, self).params
+
     def best_match_content_type(self):
         """Determine the requested response content-type."""
         supported = ('application/json',)
@@ -972,23 +983,70 @@ class Request(webob.Request):
         langs = i18n.get_available_languages('glance')
         return self.accept_language.best_match(langs)
 
-    def get_content_range(self, image_size):
+    def get_range_from_request(self, image_size):
         """Return the `Range` in a request."""
-        range_str = self.headers.get('Content-Range')
+
+        range_str = self.headers.get('Range')
         if range_str is not None:
-            range_ = webob.byterange.ContentRange.parse(range_str)
-            # NOTE(dharinic): Ensure that a range like 1-4/* for an image
-            # size of 3 is invalidated.
-            if range_ is None or (range_.length is None and
-                                  range_.stop > image_size):
-                msg = _('Malformed Content-Range header: %s') % range_str
-                raise webob.exc.HTTPRequestRangeNotSatisfiable(explanation=msg)
+
+            # NOTE(dharinic): We do not support multi range requests.
+            if ',' in range_str:
+                msg = ("Requests with multiple ranges are not supported in "
+                       "Glance. You may make multiple single-range requests "
+                       "instead.")
+                raise webob.exc.HTTPBadRequest(explanation=msg)
+
+            range_ = webob.byterange.Range.parse(range_str)
+            if range_ is None:
+                msg = ("Invalid Range header.")
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
+            # NOTE(dharinic): Ensure that a range like bytes=4- for an image
+            # size of 3 is invalidated as per rfc7233.
+            if range_.start >= image_size:
+                msg = ("Invalid start position in Range header. "
+                       "Start position MUST be in the inclusive range [0, %s]."
+                       % (image_size - 1))
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
             return range_
 
+        # NOTE(dharinic): For backward compatibility reasons, we maintain
+        # support for 'Content-Range' in requests even though it's not
+        # correct to use it in requests..
+        c_range_str = self.headers.get('Content-Range')
+        if c_range_str is not None:
+            content_range = webob.byterange.ContentRange.parse(c_range_str)
+            # NOTE(dharinic): Ensure that a content range like 1-4/* for an
+            # image size of 3 is invalidated.
+            if content_range is None:
+                msg = ("Invalid Content-Range header.")
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
+            if (content_range.length is None and
+                    content_range.stop > image_size):
+                msg = ("Invalid stop position in Content-Range header. "
+                       "The stop position MUST be in the inclusive range "
+                       "[0, %s]." % (image_size - 1))
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
+            if content_range.start >= image_size:
+                msg = ("Invalid start position in Content-Range header. "
+                       "Start position MUST be in the inclusive range [0, %s]."
+                       % (image_size - 1))
+                raise webob.exc.HTTPRequestRangeNotSatisfiable(msg)
+            return content_range
+
 
 class JSONRequestDeserializer(object):
     valid_transfer_encoding = frozenset(['chunked', 'compress', 'deflate',
                                          'gzip', 'identity'])
+    httpverb_may_have_body = frozenset({'POST', 'PUT', 'PATCH'})
+
+    @classmethod
+    def is_valid_encoding(cls, request):
+        request_encoding = request.headers.get('transfer-encoding', '').lower()
+        return request_encoding in cls.valid_transfer_encoding
+
+    @classmethod
+    def is_valid_method(cls, request):
+        return request.method.upper() in cls.httpverb_may_have_body
 
     def has_body(self, request):
         """
@@ -996,11 +1054,12 @@ class JSONRequestDeserializer(object):
 
         :param request:  Webob.Request object
         """
-        request_encoding = request.headers.get('transfer-encoding', '').lower()
-        is_valid_encoding = request_encoding in self.valid_transfer_encoding
-        if is_valid_encoding and request.is_body_readable:
+
+        if self.is_valid_encoding(request) and self.is_valid_method(request):
+            request.is_body_readable = True
             return True
-        elif request.content_length is not None and request.content_length > 0:
+
+        if request.content_length is not None and request.content_length > 0:
             return True
 
         return False
