@@ -18,6 +18,7 @@ import copy
 import functools
 import uuid
 
+from oslo_config import cfg
 from oslo_log import log as logging
 import six
 
@@ -27,7 +28,7 @@ from glance.common import utils
 from glance.db import utils as db_utils
 from glance.i18n import _, _LI, _LW
 
-
+CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 DATA = {
@@ -43,12 +44,6 @@ DATA = {
     'locations': [],
     'tasks': {},
     'task_info': {},
-    'artifacts': {},
-    'artifact_properties': {},
-    'artifact_tags': {},
-    'artifact_dependencies': {},
-    'artifact_blobs': {},
-    'artifact_blob_locations': {}
 }
 
 INDEX = 0
@@ -70,6 +65,15 @@ def log_call(func):
     return wrapped
 
 
+def configure():
+    if CONF.workers not in [0, 1]:
+        msg = _('CONF.workers should be set to 0 or 1 when using the '
+                'db.simple.api backend. Fore more info, see '
+                'https://bugs.launchpad.net/glance/+bug/1619508')
+        LOG.critical(msg)
+        raise SystemExit(msg)
+
+
 def reset():
     global DATA
     DATA = {
@@ -85,7 +89,6 @@ def reset():
         'locations': [],
         'tasks': {},
         'task_info': {},
-        'artifacts': {}
     }
 
 
@@ -832,53 +835,10 @@ def image_tag_delete(context, image_id, value):
         raise exception.NotFound()
 
 
-def is_image_mutable(context, image):
-    """Return True if the image is mutable in this context."""
-    # Is admin == image mutable
-    if context.is_admin:
-        return True
-
-    # No owner == image not mutable
-    if image['owner'] is None or context.owner is None:
-        return False
-
-    # Image only mutable by its owner
-    return image['owner'] == context.owner
-
-
 def is_image_visible(context, image, status=None):
-    """Return True if the image is visible in this context."""
-    # Is admin == image visible
-    if context.is_admin:
-        return True
-
-    # No owner == image visible
-    if image['owner'] is None:
-        return True
-
-    # Public or Community visibility == image visible
-    if image['visibility'] in ['public', 'community']:
-        return True
-
-    # Perform tests based on whether we have an owner
-    if context.owner is not None:
-        if context.owner == image['owner']:
-            return True
-
-        # Figure out if this image is shared with that tenant
-        if status == 'all':
-            status = None
-
-        if 'shared' == image['visibility']:
-            members = image_member_find(context,
-                                        image_id=image['id'],
-                                        member=context.owner,
-                                        status=status)
-            if members:
-                return True
-
-    # Private image
-    return False
+    if status == 'all':
+        status = None
+    return db_utils.is_image_visible(context, image, image_member_find, status)
 
 
 def user_get_storage_usage(context, owner_id, image_id=None, session=None):
@@ -1260,7 +1220,7 @@ def metadef_namespace_get_all(context,
                               filters=None):
     """Get a namespaces list"""
     resource_types = filters.get('resource_types', []) if filters else []
-    visibility = filters.get('visibility', None) if filters else None
+    visibility = filters.get('visibility') if filters else None
 
     namespaces = []
     for namespace in DATA['metadef_namespaces']:
@@ -1982,96 +1942,6 @@ def metadef_tag_count(context, namespace_name):
     return count
 
 
-def _artifact_format(artifact_id, **values):
-    dt = timeutils.utcnow()
-    artifact = {
-        'id': artifact_id,
-        'type_name': None,
-        'type_version_prefix': None,
-        'type_version_suffix': None,
-        'type_version_meta': None,
-        'version_prefix': None,
-        'version_suffix': None,
-        'version_meta': None,
-        'description': None,
-        'visibility': None,
-        'state': None,
-        'owner': None,
-        'scope': None,
-        'tags': [],
-        'properties': {},
-        'blobs': [],
-        'created_at': dt,
-        'updated_at': dt,
-        'deleted_at': None,
-        'deleted': False,
-    }
-
-    artifact.update(values)
-    return artifact
-
-
-@log_call
-def artifact_create(context, values, type_name, type_version):
-    global DATA
-    artifact_id = values.get('id', str(uuid.uuid4()))
-
-    if artifact_id in DATA['artifacts']:
-        raise exception.Duplicate()
-
-    if 'state' not in values:
-        raise exception.Invalid('state is a required attribute')
-
-    allowed_keys = set(['id',
-                        'type_name',
-                        'type_version',
-                        'name',
-                        'version',
-                        'description',
-                        'visibility',
-                        'state',
-                        'owner',
-                        'scope'])
-
-    incorrect_keys = set(values.keys()) - allowed_keys
-    if incorrect_keys:
-        raise exception.Invalid(
-            'The keys %s are not valid' % str(incorrect_keys))
-
-    artifact = _artifact_format(artifact_id, **values)
-    DATA['artifacts'][artifact_id] = artifact
-
-    return copy.deepcopy(artifact)
-
-
-def _artifact_get(context, artifact_id, type_name,
-                  type_version=None):
-    try:
-        artifact = DATA['artifacts'][artifact_id]
-        if (artifact['type_name'] != type_name or
-                (type_version is not None and
-                 artifact['type_version'] != type_version)):
-            raise KeyError
-    except KeyError:
-        LOG.info(_LI('Could not find artifact %s'), artifact_id)
-        raise exception.NotFound()
-
-    if artifact['deleted_at']:
-        LOG.info(_LI('Unable to get deleted image'))
-        raise exception.NotFound()
-
-    return artifact
-
-
-@log_call
-def artifact_get(context, artifact_id,
-                 type_name,
-                 type_version=None, session=None):
-    artifact = _artifact_get(context, artifact_id, type_name,
-                             type_version)
-    return copy.deepcopy(artifact)
-
-
 def _format_association(namespace, resource_type, association_values):
     association = {
         'namespace_id': namespace['id'],
