@@ -26,8 +26,6 @@ migration performs the following steps for every entry in the images table:
 
 Fixes bug #1081043
 """
-import types  # noqa
-
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import encodeutils
@@ -36,13 +34,9 @@ import sqlalchemy
 
 from glance.common import crypt
 from glance.common import exception
-from glance import i18n
+from glance.i18n import _, _LE, _LI, _LW
 
 LOG = logging.getLogger(__name__)
-_ = i18n._
-_LE = i18n._LE
-_LI = i18n._LI
-_LW = i18n._LW
 CONF = cfg.CONF
 
 CONF.import_opt('metadata_encryption_key', 'glance.common.config')
@@ -52,10 +46,6 @@ def upgrade(migrate_engine):
     migrate_location_credentials(migrate_engine, to_quoted=True)
 
 
-def downgrade(migrate_engine):
-    migrate_location_credentials(migrate_engine, to_quoted=False)
-
-
 def migrate_location_credentials(migrate_engine, to_quoted):
     """
     Migrate location credentials for encrypted swift uri's between the
@@ -162,7 +152,9 @@ def legacy_parse_uri(uri, to_quote):
         raise exception.BadStoreUri(message=reason)
 
     pieces = urlparse.urlparse(uri)
-    assert pieces.scheme in ('swift', 'swift+http', 'swift+https')
+    if pieces.scheme not in ('swift', 'swift+http', 'swift+https'):
+        raise exception.BadStoreUri(message="Unacceptable scheme: '%s'" %
+                                    pieces.scheme)
     scheme = pieces.scheme
     netloc = pieces.netloc
     path = pieces.path.lstrip('/')
