@@ -18,14 +18,16 @@ __all__ = [
     'list_scrubber_opts',
     'list_cache_opts',
     'list_manage_opts',
-    'list_artifacts_opts'
 ]
 
 import copy
 import itertools
 
+from osprofiler import opts as profiler
+
 import glance.api.middleware.context
 import glance.api.versions
+import glance.async.flows.convert
 import glance.async.taskflow_executor
 import glance.common.config
 import glance.common.location_strategy
@@ -53,6 +55,7 @@ _api_opts = [
         glance.common.wsgi.bind_opts,
         glance.common.wsgi.eventlet_opts,
         glance.common.wsgi.socket_opts,
+        glance.common.wsgi.wsgi_opts,
         glance.image_cache.drivers.sqlite.sqlite_opts,
         glance.image_cache.image_cache_opts,
         glance.notifier.notifier_opts,
@@ -63,11 +66,12 @@ _api_opts = [
         glance.scrubber.scrubber_opts))),
     ('image_format', glance.common.config.image_format_opts),
     ('task', glance.common.config.task_opts),
-    ('taskflow_executor',
-     glance.async.taskflow_executor.taskflow_executor_opts),
+    ('taskflow_executor', list(itertools.chain(
+        glance.async.taskflow_executor.taskflow_executor_opts,
+        glance.async.flows.convert.convert_task_opts))),
     ('store_type_location_strategy',
      glance.common.location_strategy.store_type.store_type_opts),
-    ('profiler', glance.common.wsgi.profiler_opts),
+    profiler.list_opts()[0],
     ('paste_deploy', glance.common.config.paste_deploy_opts)
 ]
 _registry_opts = [
@@ -76,8 +80,9 @@ _registry_opts = [
         glance.common.config.common_opts,
         glance.common.wsgi.bind_opts,
         glance.common.wsgi.socket_opts,
+        glance.common.wsgi.wsgi_opts,
         glance.common.wsgi.eventlet_opts))),
-    ('profiler', glance.common.wsgi.profiler_opts),
+    profiler.list_opts()[0],
     ('paste_deploy', glance.common.config.paste_deploy_opts)
 ]
 _scrubber_opts = [
@@ -96,22 +101,12 @@ _cache_opts = [
         glance.image_cache.drivers.sqlite.sqlite_opts,
         glance.image_cache.image_cache_opts,
         glance.registry.registry_addr_opts,
+        glance.registry.client.registry_client_opts,
         glance.registry.client.registry_client_ctx_opts))),
 ]
 _manage_opts = [
     (None, [])
 ]
-_artifacts_opts = [
-    (None, list(itertools.chain(
-        glance.api.middleware.context.context_opts,
-        glance.api.versions.versions_opts,
-        glance.common.wsgi.bind_opts,
-        glance.common.wsgi.eventlet_opts,
-        glance.common.wsgi.socket_opts,
-        glance.common.wsgi.profiler_opts,
-        glance.notifier.notifier_opts))),
-    ('paste_deploy', glance.common.config.paste_deploy_opts)
-]
 
 
 def list_api_opts():
@@ -158,8 +153,3 @@ def list_cache_opts():
 def list_manage_opts():
     """Return a list of oslo_config options available in Glance manage."""
     return [(g, copy.deepcopy(o)) for g, o in _manage_opts]
-
-
-def list_artifacts_opts():
-    """Return a list of oslo_config options available in Glance artifacts"""
-    return [(g, copy.deepcopy(o)) for g, o in _artifacts_opts]
