@@ -15,6 +15,10 @@
 
 """Common utilities used in testing"""
 
+# NOTE(rosmaita): mriedem says this is needed for importing from fixtures,
+# and he's right!
+from __future__ import absolute_import
+
 import errno
 import functools
 import os
